#!ruby -Ke
#
#  TOPPERS/A-WDG
#      Automotive Watchdog Driver
#
#  Copyright (C) 2014-2015 by Center for Embedded Computing Systems
#                             Graduate School of Information Science, Nagoya Univ., JAPAN
#  Copyright (C) 2014-2015 by AISIN COMCRUISE Co., Ltd., JAPAN
#  Copyright (C) 2014-2015 by FUJI SOFT INCORPORATED, JAPAN
#  Copyright (C) 2014-2015 by NEC Communication Systems, Ltd., JAPAN
#  Copyright (C) 2014-2015 by Panasonic Advanced Technology Development Co., Ltd., JAPAN
#  Copyright (C) 2014-2015 by SCSK Corporation, JAPAN
#  Copyright (C) 2014-2015 by Sunny Giken Inc., JAPAN
#  Copyright (C) 2014-2015 by TOSHIBA CORPORATION, JAPAN
#  Copyright (C) 2014-2015 by Witz Corporation
#
#  本ソフトウェアは，車載制御システム向け高品質プラットフォームに関す
#  るコンソーシアム型共同研究において開発したものである．本ソフトウェ
#  アの著作権は，上記の著作権者に帰属する．
#
#  $Id: check_spec_coverage.rb 3492 2017-03-14 05:45:23Z suzuki-kawaguchi $
#

#
#  仕様カバレッジをチェックする
#

if ($0 == __FILE__)
  TOOL_ROOT = File.expand_path(File.dirname(__FILE__) + "/")
  $LOAD_PATH.unshift(TOOL_ROOT)
end

require "pp"

# 対象タグ正規表現
TAG_REG_ALL = "N?D?WDGa?[0-9][0-9][0-9][_Conf]*"

# WDGタグ一覧
aWdgSpecId = File.read("#{TOOL_ROOT}/wdg_spec_id.txt").split("\n")

# 実装済みタグ一覧ファイル
aImplSpecId = []
aFileData = File.read("#{TOOL_ROOT}/wdg_impl_id.txt").split("\n")
aFileData.each{|sLine|
  if (sLine =~ /\W#{TAG_REG_ALL}\W/)
    aImplSpecId.push(sLine.gsub(/^\W(#{TAG_REG_ALL})\W.*$/, "\\1"))
  end
}

# 未実装タグ一覧
aNonImplTags = []
aWdgSpecId.each{|sTag|
  if (!aImplSpecId.include?(sTag))
    aNonImplTags.push(sTag)
  end
}

# 結果表示
puts("=" * 30)
puts("Wdg spec tags   : #{aWdgSpecId.size()}")
puts("Implemented tags: #{aImplSpecId.size()}")
fCoverage = aImplSpecId.size().to_f() / aWdgSpecId.size().to_f() * 100
puts("Spec Coverage   : #{sprintf('%0.1f', fCoverage)} %")
puts("-" * 30)
puts("Not implementation tags: #{aNonImplTags.size()}")
puts("=" * 30)

aNonImplTags.each{|sTag|
  puts(sTag)
}

# 仕様にないタグが実装されていないかチェック
 aImplSpecId.each{|sTag|
  if (!aWdgSpecId.include?(sTag))
    puts("!! Not exist tag: #{sTag}")
  end
}

