#!ruby -Ke
#
#  TOPPERS/A-WDG
#      Automotive Watchdog Driver
#
#  Copyright (C) 2014-2015 by Center for Embedded Computing Systems
#                             Graduate School of Information Science, Nagoya Univ., JAPAN
#  Copyright (C) 2014-2015 by AISIN COMCRUISE Co., Ltd., JAPAN
#  Copyright (C) 2014-2015 by FUJI SOFT INCORPORATED, JAPAN
#  Copyright (C) 2014-2015 by NEC Communication Systems, Ltd., JAPAN
#  Copyright (C) 2014-2015 by Panasonic Advanced Technology Development Co., Ltd., JAPAN
#  Copyright (C) 2014-2015 by SCSK Corporation, JAPAN
#  Copyright (C) 2014-2015 by Sunny Giken Inc., JAPAN
#  Copyright (C) 2014-2015 by TOSHIBA CORPORATION, JAPAN
#  Copyright (C) 2014-2015 by Witz Corporation
#
#  本ソフトウェアは，車載制御システム向け高品質プラットフォームに関す
#  るコンソーシアム型共同研究において開発したものである．本ソフトウェ
#  アの著作権は，上記の著作権者に帰属する．
#
#  $Id: get_spec_id.rb 3492 2017-03-14 05:45:23Z suzuki-kawaguchi $
#

#=====================================================================
# テキストファイル(wdg_spec.txt)から，WDGに関する仕様タグを抜き出して，
# ID一覧をファイルへ出力する
# ※テキストファイルは，外部仕様書をWord等の変換機能を使用して作成する
#=====================================================================

# 書き出すファイル
sWriteFileId = File.dirname(__FILE__) + "/wdg_spec_id.txt"

# pdfから読み出したテキストデータ
sTextFile = "wdg_spec.txt"
sTextSpe = "../doc/wdg_spec.txt"
sTextDes = "../doc/wdg_design.txt"

# テキストファイルを読み込む
sText = File.read(sTextFile) + File.read(sTextSpe) + File.read(sTextDes)

# 改行を削除(タグの途中で改行が入るケース対応)
sText.gsub!("\r\n", "")
sText.gsub!("\r", "")
sText.gsub!("\n", "")

# 対象タグを検索する
aData1 = sText.scan(/\[WDG[0-9][0-9][0-9]\]/)
aData2 = sText.scan(/WDG[0-9][0-9][0-9]_Conf/)
aData3 = sText.scan(/WDGa[0-9][0-9][0-9]/)
aData4 = sText.scan(/NWDG[0-9][0-9][0-9]/)
aData5 = sText.scan(/DWDG[0-9][0-9][0-9]/)

# 削除された仕様タグ
aDelTag = []

# 不要な文字列を削除
aTmp = []
aData1.each{|sTag|
  if (sTag.include?("＜"))
    aDelTag.push(sTag.gsub(/\[(WDG[0-9][0-9][0-9])\]/, "\\1"))
  else
    aTmp.push(sTag.gsub(/\[(WDG[0-9][0-9][0-9])\]/, "\\1"))
  end
}
aData1 = aTmp

# 重複したIDを削除
aData1.uniq!
aData2.uniq!
aData3.uniq!
aData4.uniq!
aData5.uniq!

# タグの数を計測
nData1 = aData1.size()
nData2 = aData2.size()
nData3 = aData3.size()
nData4 = aData4.size()
nData5 = aData5.size()

# タグ一覧を出力
aData = aData1 + aData2 + aData3 + aData4 + aData5

# IDでソート
aData.sort!

# 1つの文字列にする
sOutString = aData.join("\n")

# ファイルへ出力する
File.open(sWriteFileId, "w") {|io|
  io.puts(sOutString)
}

# 仕様IDの件数を表示する
puts("=" * 20)
printf("WDG***      : %3d\n", nData1)
printf("WDG***_Conf : %3d\n", nData2)
printf("WDGa***     : %3d\n", nData3)
printf("NWDG***     : %3d\n", nData4)
printf("DWDG***     : %3d\n", nData5)
puts("-" * 20)
printf("Total       : %3d\n", (nData1 + nData2 + nData3 + nData4 + nData5))
puts("=" * 20)
printf("Delete tags : %3d\n", aDelTag.size())
if (aDelTag != nil)
  aDelTag.sort!
  puts("-" * 20)
  aDelTag.each{|sDelTag|
    puts(sDelTag)
  }
end
puts("=" * 20)
