/*
 *  TOPPERS/A-WDG
 *      Automotive Watchdog Driver
 *
 *  Copyright (C) 2014-2015 by Center for Embedded Computing Systems
 *                             Graduate School of Information Science, Nagoya Univ., JAPAN
 *  Copyright (C) 2014-2015 by AISIN COMCRUISE Co., Ltd., JAPAN
 *  Copyright (C) 2014-2015 by FUJI SOFT INCORPORATED, JAPAN
 *  Copyright (C) 2014-2015 by NEC Communication Systems, Ltd., JAPAN
 *  Copyright (C) 2014-2015 by Panasonic Advanced Technology Development Co., Ltd., JAPAN
 *  Copyright (C) 2014-2015 by SCSK Corporation, JAPAN
 *  Copyright (C) 2014-2015 by Sunny Giken Inc., JAPAN
 *  Copyright (C) 2014-2015 by TOSHIBA CORPORATION, JAPAN
 *  Copyright (C) 2014-2015 by Witz Corporation
 *
 *  上記著作権者は，以下の(1)〜(4)の条件を満たす場合に限り，本ソフトウェ
 *  ア（本ソフトウェアを改変したものを含む．以下同じ）を使用・複製・改
 *  変・再配布（以下，利用と呼ぶ）することを無償で許諾する．
 *  (1) 本ソフトウェアをソースコードの形で利用する場合には，上記の著作
 *      権表示，この利用条件および下記の無保証規定が，そのままの形でソー
 *      スコード中に含まれていること．
 *  (2) 本ソフトウェアを，ライブラリ形式など，他のソフトウェア開発に使
 *      用できる形で再配布する場合には，再配布に伴うドキュメント（利用
 *      者マニュアルなど）に，上記の著作権表示，この利用条件および下記
 *      の無保証規定を掲載すること．
 *  (3) 本ソフトウェアを，機器に組み込むなど，他のソフトウェア開発に使
 *      用できない形で再配布する場合には，次のいずれかの条件を満たすこ
 *      と．
 *    (a) 再配布に伴うドキュメント（利用者マニュアルなど）に，上記の著
 *        作権表示，この利用条件および下記の無保証規定を掲載すること．
 *    (b) 再配布の形態を，別に定める方法によって，TOPPERSプロジェクトに
 *        報告すること．
 *  (4) 本ソフトウェアの利用により直接的または間接的に生じるいかなる損
 *      害からも，上記著作権者およびTOPPERSプロジェクトを免責すること．
 *      また，本ソフトウェアのユーザまたはエンドユーザからのいかなる理
 *      由に基づく請求からも，上記著作権者およびTOPPERSプロジェクトを
 *      免責すること．
 *
 *  本ソフトウェアは，AUTOSAR（AUTomotive Open System ARchitecture）仕
 *  様に基づいている．上記の許諾は，AUTOSARの知的財産権を許諾するもので
 *  はない．AUTOSARは，AUTOSAR仕様に基づいたソフトウェアを商用目的で利
 *  用する者に対して，AUTOSARパートナーになることを求めている．
 *
 *  本ソフトウェアは，無保証で提供されているものである．上記著作権者お
 *  よびTOPPERSプロジェクトは，本ソフトウェアに関して，特定の使用目的
 *  に対する適合性も含めて，いかなる保証も行わない．また，本ソフトウェ
 *  アの利用により直接的または間接的に生じたいかなる損害に関しても，そ
 *  の責任を負わない．
 *
 *  $Id: Wdg_Target.h 3492 2017-03-14 05:45:23Z suzuki-kawaguchi $
 */

#ifndef WDG_TARGET_H
#define WDG_TARGET_H

#include "SchM_Wdg.h"
#include "prc_sil.h"
#include "Std_Types.h"
#include "Wdg.h"
#include "target_hw_counter.h"

/* トリガクロック */
#define TRIGGER_TIMER_CLOCK		(uint32) 8000U

/* ウォッチドッグクロック(Hz) */
#define WDTATCKI				1875U

/* ウォッチドッグタイマのベースアドレス */
#define WDGT_BASE				0xFFED0000U

/* ウォッチドッグタイマトリガ割込み番号 */
#define WDGT_TRIGGER_IRQ		TAUFJ0I2_INTNO

/* ウォッチドッグタイマトリガのベースユニット */
#define WDGT_TRIGGER_U			0U

/* ウォッチドッグタイマトリガのベースチャネル */
#define WDGT_TRIGGER_CH			2U

#define WDTA_WDTE				0x00U
#define WDTA_MD					0x0CU

/* ビット定義 */
#define WDTA_RUN				0xACU

#define WDTA_OVF_9P				0x00U
#define WDTA_OVF_10P			0x10U
#define WDTA_OVF_11P			0x20U
#define WDTA_OVF_12P			0x30U
#define WDTA_OVF_13P			0x40U
#define WDTA_OVF_14P			0x50U
#define WDTA_OVF_15P			0x60U
#define WDTA_OVF_16P			0x70U

#define WDTA_ERM_NMI			0x00U
#define WDTA_ERM_RESET			0x04U

#define WDTA_WS_25				0x00U
#define WDTA_WS_50				0x01U
#define WDTA_WS_75				0x02U
#define WDTA_WS_100				0x03U

/* 非依存部定義 */
/* ウィンドウオープン期間(%) */
#define WINDOW_OPEN_25			25U
#define WINDOW_OPEN_50			50U
#define WINDOW_OPEN_75			75U
#define WINDOW_OPEN_100			100U

/* HWタイムアウト時間(ms) */
#define HW_TIMEOUT_273			273U
#define HW_TIMEOUT_546			546U
#define HW_TIMEOUT_1092			1092U
#define HW_TIMEOUT_2184			2184U
#define HW_TIMEOUT_4369			4369U
#define HW_TIMEOUT_8738			8738U
#define HW_TIMEOUT_17476		17476U
#define HW_TIMEOUT_34952		34952U

/* 内部関数プロトタイプ宣言 */
LOCAL_INLINE void wdg_trigger_init(uint16 wdg_interrupt_period);
LOCAL_INLINE void wdg_timer_init(const WDG_MODE_INIB *p_current_mode);
LOCAL_INLINE void wdg_trigger_int_clear(void);
LOCAL_INLINE void wdg_timer_trigger(const WDG_MODE_INIB *p_current_mode);
LOCAL_INLINE void wdg_target_start(const WDG_MODE_INIB *p_current_mode);
LOCAL_INLINE void wdg_target_off_mode(void);
LOCAL_INLINE void wdg_target_set_mode(const WDG_MODE_INIB *p_current_mode);
LOCAL_INLINE uint16 wdg_target_reset_timeout(uint16 trigger_cnt, uint16 pre_interrupt_period);

/*
 *  ウォッチドッグタイマトリガ割込みの初期化処理
 */
LOCAL_INLINE void
wdg_trigger_init(uint16 wdg_interrupt_period)
{
	/* タイマ停止 */
	SetTimerStopTAUJ(WDGT_TRIGGER_U, WDGT_TRIGGER_CH);

	/* 割込み要求クリア */
	HwcounterClearInterrupt(TAUJ_INTNO(WDGT_TRIGGER_U, WDGT_TRIGGER_CH));

	/* 割込み許可 */
	HwcounterEnableInterrupt(TAUJ_INTNO(WDGT_TRIGGER_U, WDGT_TRIGGER_CH));

	/* インターバルタイマ設定 */
	sil_wrh_mem((void *) TAUJCMOR(WDGT_TRIGGER_U, WDGT_TRIGGER_CH), MCU_TAUJ00_CMOR);
	sil_wrb_mem((void *) TAUJCMUR(WDGT_TRIGGER_U, WDGT_TRIGGER_CH), MCU_TAUJ00_CMUR);

	/* タイマカウント周期設定 */
	sil_wrw_mem((void *) TAUJCDR(WDGT_TRIGGER_U, WDGT_TRIGGER_CH), (wdg_interrupt_period * TRIGGER_TIMER_CLOCK));

	/* タイマ動作開始 */
	SetTimerStartTAUJ(WDGT_TRIGGER_U, WDGT_TRIGGER_CH);

	return;
}

/*
 *  [WDG161] ウォッチドッグタイマ初期化処理
 */
LOCAL_INLINE void
wdg_timer_init(const WDG_MODE_INIB *p_current_mode)
{
	uint8	wdta_ovf;
	uint8	wdta_md = 0U;

	/* wdta_ovfリセット値参照 */
	wdta_ovf = sil_reb_mem((void *) (WDGT_BASE + WDTA_MD));
	wdta_ovf &= 0x70U;

	/* タイムアウト時動作設定 */
	switch (p_current_mode->WdgTimeoutReaction) {
	case WDG_RESET:
		wdta_md |= WDTA_ERM_RESET;
		break;
	case WDG_NMI:
		wdta_md |= WDTA_ERM_NMI;
		break;
	default:
		wdta_md |= WDTA_ERM_RESET;
		break;
	}

	/* ウィンドウオープン期間設定 */
	switch (p_current_mode->WdgWindowOpenRate) {
	case WINDOW_OPEN_100:
		wdta_md |= WDTA_WS_100;
		break;
	case WINDOW_OPEN_75:
		wdta_md |= WDTA_WS_75;
		break;
	case WINDOW_OPEN_50:
		wdta_md |= WDTA_WS_50;
		break;
	case WINDOW_OPEN_25:
		wdta_md |= WDTA_WS_25;
		break;
	default:
		wdta_md |= WDTA_WS_100;
		break;
	}

	/* オーバフローインターバル時間設定 */
	switch (p_current_mode->timeout) {
	case HW_TIMEOUT_34952:
		wdta_md |= WDTA_OVF_16P;
		break;
	case HW_TIMEOUT_17476:
		wdta_md |= WDTA_OVF_15P;
		break;
	case HW_TIMEOUT_8738:
		wdta_md |= WDTA_OVF_14P;
		break;
	case HW_TIMEOUT_4369:
		wdta_md |= WDTA_OVF_13P;
		break;
	case HW_TIMEOUT_2184:
		wdta_md |= WDTA_OVF_12P;
		break;
	case HW_TIMEOUT_1092:
		wdta_md |= WDTA_OVF_11P;
		break;
	case HW_TIMEOUT_546:
		wdta_md |= WDTA_OVF_10P;
		break;
	case HW_TIMEOUT_273:
		wdta_md |= WDTA_OVF_9P;
		break;
	default:
		wdta_md |= wdta_ovf;
		break;
	}

	sil_wrb_mem((void *) (WDGT_BASE + WDTA_MD), wdta_md);

	return;
}

/*
 *  ウォッチドッグタイマトリガ割込み要求クリア
 */
LOCAL_INLINE void
wdg_trigger_int_clear(void)
{
	/* 割込み要求クリア */
	HwcounterClearInterrupt(TAUJ_INTNO(WDGT_TRIGGER_U, WDGT_TRIGGER_CH));

	return;
}

/*
 *  [WDG161] ウォッチドッグタイマトリガ処理
 */
LOCAL_INLINE void
wdg_timer_trigger(const WDG_MODE_INIB *p_current_mode)
{
	/* タイマトリガ */
	sil_wrb_mem((void *) (WDGT_BASE + WDTA_WDTE), WDTA_RUN);

	return;
}

/*
 *  ウォッチドッグ依存部初期化処理
 */
LOCAL_INLINE void
wdg_target_start(const WDG_MODE_INIB *p_current_mode)
{
	/* タイマ開始 */
	wdg_timer_init(p_current_mode);

	/* タイマトリガ */
	sil_wrb_mem((void *) (WDGT_BASE + WDTA_WDTE), WDTA_RUN);

	/* タイマトリガ割込み開始 */
	wdg_trigger_init(p_current_mode->interrupt_period);

	return;
}

/*
 *  ウォッチドッグ依存部offモード設定
 */
LOCAL_INLINE void
wdg_target_off_mode(void)
{
	return;
}

/*
 *  ウォッチドッグ依存部モード設定(本マイコンは未対応)
 */
LOCAL_INLINE void
wdg_target_set_mode(const WDG_MODE_INIB *p_current_mode)
{
	return;
}

/*
 *  タイムアウト再設定(本マイコンは未対応)
 */
LOCAL_INLINE uint16
wdg_target_reset_timeout(uint16 trigger_cnt, uint16 pre_interrupt_period)
{
	uint16 timeout = 0U;

	return(timeout);
}

/*
 *  未サポート
 *  - アクティベーションコード
 *    <WDG093><WDG094><WDG095>
 *  - トリガロケーション(メモリアドレス)設定
 *    <WDG034>
 *  - ランエリア(ROM or RAM)選択
 *    <WDG168>
 *  - 非依存部との分離のため
 *    <WDG031>
 */

#endif /* WDG_TARGET_H */
