/*
 *  TOPPERS/A-WDGIF
 *      AuTomotive Watchdog Interface
 *
 *  Copyright (C) 2014-2015 by Center for Embedded Computing Systems
 *                             Graduate School of Information Science, Nagoya Univ., JAPAN
 *  Copyright (C) 2014-2015 by AISIN COMCRUISE Co., Ltd., JAPAN
 *  Copyright (C) 2014-2015 by FUJI SOFT INCORPORATED, JAPAN
 *  Copyright (C) 2014-2015 by NEC Communication Systems, Ltd., JAPAN
 *  Copyright (C) 2014-2015 by Panasonic Advanced Technology Development Co., Ltd., JAPAN
 *  Copyright (C) 2014-2015 by SCSK Corporation, JAPAN
 *  Copyright (C) 2014-2015 by Sunny Giken Inc., JAPAN
 *  Copyright (C) 2014-2015 by TOSHIBA CORPORATION, JAPAN
 *  Copyright (C) 2014-2015 by Witz Corporation
 *
 *  上記著作権者は，以下の(1)〜(4)の条件を満たす場合に限り，本ソフトウェ
 *  ア（本ソフトウェアを改変したものを含む．以下同じ）を使用・複製・改
 *  変・再配布（以下，利用と呼ぶ）することを無償で許諾する．
 *  (1) 本ソフトウェアをソースコードの形で利用する場合には，上記の著作
 *      権表示，この利用条件および下記の無保証規定が，そのままの形でソー
 *      スコード中に含まれていること．
 *  (2) 本ソフトウェアを，ライブラリ形式など，他のソフトウェア開発に使
 *      用できる形で再配布する場合には，再配布に伴うドキュメント（利用
 *      者マニュアルなど）に，上記の著作権表示，この利用条件および下記
 *      の無保証規定を掲載すること．
 *  (3) 本ソフトウェアを，機器に組み込むなど，他のソフトウェア開発に使
 *      用できない形で再配布する場合には，次のいずれかの条件を満たすこ
 *      と．
 *    (a) 再配布に伴うドキュメント（利用者マニュアルなど）に，上記の著
 *        作権表示，この利用条件および下記の無保証規定を掲載すること．
 *    (b) 再配布の形態を，別に定める方法によって，TOPPERSプロジェクトに
 *        報告すること．
 *  (4) 本ソフトウェアの利用により直接的または間接的に生じるいかなる損
 *      害からも，上記著作権者およびTOPPERSプロジェクトを免責すること．
 *      また，本ソフトウェアのユーザまたはエンドユーザからのいかなる理
 *      由に基づく請求からも，上記著作権者およびTOPPERSプロジェクトを
 *      免責すること．
 *
 *  本ソフトウェアは，AUTOSAR（AUTomotive Open System ARchitecture）仕
 *  様に基づいている．上記の許諾は，AUTOSARの知的財産権を許諾するもので
 *  はない．AUTOSARは，AUTOSAR仕様に基づいたソフトウェアを商用目的で利
 *  用する者に対して，AUTOSARパートナーになることを求めている．
 *
 *  本ソフトウェアは，無保証で提供されているものである．上記著作権者お
 *  よびTOPPERSプロジェクトは，本ソフトウェアに関して，特定の使用目的
 *  に対する適合性も含めて，いかなる保証も行わない．また，本ソフトウェ
 *  アの利用により直接的または間接的に生じたいかなる損害に関しても，そ
 *  の責任を負わない．
 *
 *  $Id: WdgIf.h 3492 2017-03-14 05:45:23Z suzuki-kawaguchi $
 */

/* [WDGIF001][WDGIF002] WdgIf.h */

/*
 *  開発方針
 *  - [WDGIF003] WdgIfの機能はWdgMからの情報をWdgへ通知する
 */

#ifndef WDGIF_H
#define WDGIF_H

/* [WDGIF034][WDGIF037][WDGIF041] */
#include "Os.h"
#include "Std_Types.h"
#include "WdgIf_Cfg.h"      /* [WDGIF050] */
#include "WdgIf_Types.h"    /* [WDGIF010][WDGIF047][WDGIF049] */

/*
 * DETへ通知するサービスID
 */
#define WdgIfServiceId_SetMode				0x01U
#define WdgIfServiceId_SetTriggerCondition	0x02U
#define WdgIfServiceId_GetVersionInfo		0x03U

/* [WDGIF006][WDGIF009][WDGIF030] エラーコード */
#define WDGIF_E_PARAM_DEVICE				0x01U
#define WDGIF_E_INV_POINTER					0x02U

/* WDGIFモジュールID */
#define WDGIF_MODULE_ID						0x2bU

/* [WDGIF005] バージョン情報 */
#define WDGIF_AR_RELEASE_MAJOR_VERSION		4U
#define WDGIF_AR_RELEASE_MINOR_VERSION		0U
#define WDGIF_AR_RELEASE_REVISION_VERSION	3U

/* サプライヤバージョン情報 */
#define WDGIF_SW_MAJOR_VERSION				1U
#define WDGIF_SW_MINOR_VERSION				1U
#define WDGIF_SW_PATCH_VERSION				1U
#define WDGIF_VENDOR_ID						65U	/* NCESベンダID */

/*
 *  [WDGIF007][WDGIF031][WDGIF032][WDGIF048] チェックマクロ
 */
#if WDGIF_DEV_ERROR_DETECT != FALSE
#define WDGIF_CHECK_PARAM(exp, ApiId, ErrorId)										\
	do {																			\
		if (!(exp)) {																\
			(void) Det_ReportError(WDGIF_MODULE_ID, 0x00U, (ApiId), (ErrorId));		\
			goto error_exit;														\
		}																			\
	} while (0)

#define WDGIF_CHECK_PARAM_ERCD(exp, ApiId, ErrorId)									\
	do {																			\
		if (!(exp)) {																\
			ercd = E_NOT_OK;														\
			(void) Det_ReportError(WDGIF_MODULE_ID, 0x00U, (ApiId), (ErrorId));		\
			goto error_exit;														\
		}																			\
	} while (0)
#else /* WDGIF_DEV_ERROR_DETECT == FALSE */
#define WDGIF_CHECK_PARAM(exp, ApiId, ErrorId)
#define WDGIF_CHECK_PARAM_ERCD(exp, ApiId, ErrorId)
#endif /* WDGIF_DEV_ERROR_DETECT != FALSE */

typedef uint8 WdgIf_ModeType;

/*
 *  [WDGIF013][WDGIF017] WdgIfシステムサービスプロトタイプ宣言
 */
extern Std_ReturnType WdgIf_SetMode(uint8 DeviceIndex, WdgIf_ModeType WdgMode);
extern void WdgIf_SetTriggerCondition(uint8 DeviceIndex, uint16 Timeout);
extern void WdgIf_GetVersionInfo(Std_VersionInfoType *VersionInfoPtr);

/*
 * デバッグ
 * [WDGIF052][WDGIF053][WDGIF054][WDGIF055]
 */

/*
 * 単一のWdgのみに対応するため，以下の仕様は未対応
 * <WDGIF020>
 */

#endif /* WDGIF_H */
