/*
 *  TOPPERS/A-WDG
 *      Automotive Watchdog Driver
 *
 *  Copyright (C) 2014-2015 by Center for Embedded Computing Systems
 *                             Graduate School of Information Science, Nagoya Univ., JAPAN
 *  Copyright (C) 2014-2015 by AISIN COMCRUISE Co., Ltd., JAPAN
 *  Copyright (C) 2014-2015 by FUJI SOFT INCORPORATED, JAPAN
 *  Copyright (C) 2014-2015 by NEC Communication Systems, Ltd., JAPAN
 *  Copyright (C) 2014-2015 by Panasonic Advanced Technology Development Co., Ltd., JAPAN
 *  Copyright (C) 2014-2015 by SCSK Corporation, JAPAN
 *  Copyright (C) 2014-2015 by Sunny Giken Inc., JAPAN
 *  Copyright (C) 2014-2015 by TOSHIBA CORPORATION, JAPAN
 *  Copyright (C) 2014-2015 by Witz Corporation
 *
 *  上記著作権者は，以下の(1)〜(4)の条件を満たす場合に限り，本ソフトウェ
 *  ア（本ソフトウェアを改変したものを含む．以下同じ）を使用・複製・改
 *  変・再配布（以下，利用と呼ぶ）することを無償で許諾する．
 *  (1) 本ソフトウェアをソースコードの形で利用する場合には，上記の著作
 *      権表示，この利用条件および下記の無保証規定が，そのままの形でソー
 *      スコード中に含まれていること．
 *  (2) 本ソフトウェアを，ライブラリ形式など，他のソフトウェア開発に使
 *      用できる形で再配布する場合には，再配布に伴うドキュメント（利用
 *      者マニュアルなど）に，上記の著作権表示，この利用条件および下記
 *      の無保証規定を掲載すること．
 *  (3) 本ソフトウェアを，機器に組み込むなど，他のソフトウェア開発に使
 *      用できない形で再配布する場合には，次のいずれかの条件を満たすこ
 *      と．
 *    (a) 再配布に伴うドキュメント（利用者マニュアルなど）に，上記の著
 *        作権表示，この利用条件および下記の無保証規定を掲載すること．
 *    (b) 再配布の形態を，別に定める方法によって，TOPPERSプロジェクトに
 *        報告すること．
 *  (4) 本ソフトウェアの利用により直接的または間接的に生じるいかなる損
 *      害からも，上記著作権者およびTOPPERSプロジェクトを免責すること．
 *      また，本ソフトウェアのユーザまたはエンドユーザからのいかなる理
 *      由に基づく請求からも，上記著作権者およびTOPPERSプロジェクトを
 *      免責すること．
 *
 *  本ソフトウェアは，AUTOSAR（AUTomotive Open System ARchitecture）仕
 *  様に基づいている．上記の許諾は，AUTOSARの知的財産権を許諾するもので
 *  はない．AUTOSARは，AUTOSAR仕様に基づいたソフトウェアを商用目的で利
 *  用する者に対して，AUTOSARパートナーになることを求めている．
 *
 *  本ソフトウェアは，無保証で提供されているものである．上記著作権者お
 *  よびTOPPERSプロジェクトは，本ソフトウェアに関して，特定の使用目的
 *  に対する適合性も含めて，いかなる保証も行わない．また，本ソフトウェ
 *  アの利用により直接的または間接的に生じたいかなる損害に関しても，そ
 *  の責任を負わない．
 *
 *  $Id: os.c 3492 2017-03-14 05:45:23Z suzuki-kawaguchi $
 */

#include "Os.h"
#include "sysmod/serial.h"
#include "sysmod/syslog.h"
#include "sysmod/banner.h"
#include "target_serial.h"
#include "prc_sil.h"
#include "Wdg.h"
#include "Wdg_PBcfg.h"

/* INTP4 */
#define FCLA0CTL4_INTPL	0xFFC34030U
#define INTP4_NO		35U

/* INTP5 */
#define FCLA0CTL5_INTPL	0xFFC34034U
#define INTP5_NO		36U

void target_init(void);

TASK(MainTask)
{
	target_init();

	syslog(LOG_NOTICE, "=== Wdg_Init ===");
	Wdg_Init(&wdg_config_WdgSettingsConfig_0);

	while (1) {
	}

	ShutdownOS(E_OK);
}

/*
 *  SW_INTB割り込み
 */
ISR(C2ISR_FOR_SW_INTB)
{
	uint32			u32_address;
	volatile uint32	i;

	/* 割込み制御レジスタ */
	u32_address = EIC_ADDRESS(INTP5_NO);

	/* 割込み要求ビットのクリア */
	sil_wrh_mem((void *) u32_address, sil_reh_mem((void *) u32_address) & ~EIRFn);

	/* LED3点灯 */
	fl850_led_output(0x40U);

	syslog(LOG_NOTICE, "[SW_INTB] CALL:Wdg_SetTriggerCondition(10000U)");
	/* トリガコンディション */
	Wdg_SetTriggerCondition(10000U);

	/* ウェイト */
	for (i = 0U; i < 10000U; i++) {
	}

	/* LED消灯 */
	fl850_led_output(0x00U);
}

/*
 *  SW_INTC割り込み
 */
ISR(C2ISR_FOR_SW_INTC)
{
	uint32 u32_address;

	/* 割込み制御レジスタ */
	u32_address = EIC_ADDRESS(INTP4_NO);

	/* 割込み要求ビットのクリア */
	sil_wrh_mem((void *) u32_address, sil_reh_mem((void *) u32_address) & ~EIRFn);

	syslog(LOG_NOTICE, "[SW_INTC] CALL:Wdg_SetTriggerCondition(0U)");
	/* トリガコンディションクリア */
	Wdg_SetTriggerCondition(0U);

	/* LED4点灯 */
	fl850_led_output(0x80U);
}

ProtectionReturnType
ProtectionHook(StatusType FatalError)
{
	syslog(LOG_NOTICE, "## ProtectionHook is called !! (%d)", FatalError);

	return(PRO_SHUTDOWN);
}

void
StartupHook(void)
{
	syslog_initialize();
	syslog_msk_log(LOG_UPTO(LOG_INFO));
	InitSerial();
	print_banner();
}

sint32
main(void)
{
	StartOS(MainApp);
	while (1) ;
	return(0);
}

/* ウォッチドッグNMI */
void
wdg_nmi(void)
{
	syslog(LOG_NOTICE, "## NMI");

	/* LED2点灯 */
	fl850_led_output(0x20U);
	while (1) ;
}

/* ターゲット初期化 */
void
target_init(void)
{
	uint8	u8_reg;
	uint16	u16_reg;
	uint32	u32_address;
	uint32	u32_reg;

	/* INTP4有効化 */
	u8_reg = sil_reb_mem((void *) (FCLA0CTL4_INTPL));

	sil_wrb_mem((void *) (FCLA0CTL4_INTPL), (u8_reg | 0x01U));

	u16_reg = sil_reh_mem((void *) PM(8U));
	sil_wrh_mem((void *) PM(8U), (u16_reg | 0x0001U));

	u16_reg = sil_reh_mem((void *) PFC(8U));
	sil_wrh_mem((void *) PFC(8U), (u16_reg & ~0x0001U));

	u16_reg = sil_reh_mem((void *) PFCE(8U));
	sil_wrh_mem((void *) PFCE(8), (u16_reg | 0x0001U));

	u16_reg = sil_reh_mem((void *) PFCAE(8U));
	sil_wrh_mem((void *) PFCAE(8U), (u16_reg & ~0x0001U));

	u16_reg = sil_reh_mem((void *) PMC(8U));
	sil_wrh_mem((void *) PMC(8U), (u16_reg | 0x0001U));

	u32_address = EIC_ADDRESS(INTP4_NO);
	u32_reg = sil_reh_mem((void *) u32_address);
	sil_wrh_mem((void *) u32_address, (u32_reg & ~EIRFn));
	sil_wrh_mem((void *) u32_address, (u32_reg & ~EIMKn));

	/* INTP5有効化 */
	u8_reg = sil_reb_mem((void *) (FCLA0CTL5_INTPL));

	sil_wrb_mem((void *) (FCLA0CTL5_INTPL), (u8_reg | 0x01U));

	u16_reg = sil_reh_mem((void *) PM(11U));
	sil_wrh_mem((void *) PM(11U), (u16_reg | 0x0080U));

	u16_reg = sil_reh_mem((void *) PFC(11U));
	sil_wrh_mem((void *) PFC(11U), (u16_reg & ~0x0080U));

	u16_reg = sil_reh_mem((void *) PFCE(11U));
	sil_wrh_mem((void *) PFCE(11U), (u16_reg & ~0x0080U));

	u16_reg = sil_reh_mem((void *) PFCAE(11U));
	sil_wrh_mem((void *) PFCAE(11U), (u16_reg & ~0x0080U));

	u16_reg = sil_reh_mem((void *) PMC(11U));
	sil_wrh_mem((void *) PMC(11U), (u16_reg | 0x0080U));

	u32_address = EIC_ADDRESS(INTP5_NO);
	u32_reg = sil_reh_mem((void *) u32_address);
	sil_wrh_mem((void *) u32_address, (u32_reg & ~EIRFn));
	sil_wrh_mem((void *) u32_address, (u32_reg & ~EIMKn));
}
